module Datatypes.Game (Game(..), Command(..), completed, new) where

import Data.Hashable (hash)
import Data.Set (Set)
import qualified Data.Set as Set

import Datatypes.Board (Board)
import Datatypes.Unit (Unit)
import qualified Datatypes.Unit as Unit

data Command = MoveW
             | MoveE
             | MoveSW
             | MoveSE
             | RotateClockwise
             | RotateCounterclockwise
             deriving (Show,Eq)

type UnitHash = Int

data Game = Game {
          board :: Board,
          units :: [Unit],
          visitedUnits :: Set UnitHash,
          oldLines :: Int,
          score :: Int,
          history :: [Command]
          }
          deriving Show

completed :: Game -> Bool
completed game = null $ units game

new :: Board -> [Unit] -> Game
new b us = Game {
                board = b,
                units = (c:cs),
                visitedUnits = Set.singleton (hash c),
                oldLines = 0,
                score = 0,
                history = []
                } where (c:cs) = map (flip Unit.centeredIn b) us
