{-# OPTIONS -Wall #-}

module Strategy0 where

import qualified Data.PQueue.Prio.Max as PQ

import Datatypes
import Datatypes.Game (Command(..))
import qualified Datatypes.Game as Game
import VM

import Debug.Trace (trace)

commandsList :: [Command]
commandsList = [MoveSE, MoveSW, MoveW, MoveE]

type Queue = PQ.MaxPQueue Int Game

strat0 :: Game -> ([Command],Int)
strat0 game | trace (show game) False = undefined
strat0 game = let firstQueue = PQ.singleton (Game.score game) game
                  (incomplete, completed) = findBest maxIter firstQueue []
                  (_, bestIncomplete) = PQ.findMax incomplete
                  resultGame = findListMax (bestIncomplete:completed)
                  in trace (show resultGame) (reverse (Game.history resultGame), Game.score resultGame)
                  where
                      maxIter = 30000

findListMax :: [Game] -> Game
findListMax (x:xs) = innerFindListMax x xs where
    innerFindListMax currentMax [] = currentMax
    innerFindListMax currentMax (y:ys) = innerFindListMax (if (Game.score currentMax) > (Game.score y) then currentMax else y) ys

partition :: (a -> Bool) -> [a] -> ([a], [a])
partition p items = innerPartition items [] [] where
    innerPartition [] ts fs = (ts, fs)
    innerPartition (x:xs) ts fs = if p x
                                      then innerPartition xs (x:ts) fs
                                      else innerPartition xs ts (x:fs)

findBest :: Int -> Queue -> [Game] -> (Queue, [Game])
findBest 0 queue completed = (queue, completed)
findBest i queue completed =
        let candidates = map (step game) commandsList
            (newQueue, newCompleted) = updateCollections candidates remQueue (game:completed)
            in findBest (i - 1) newQueue newCompleted
            where
                ((score, game), remQueue) = PQ.deleteFindMax queue
                updateCollections [] q l = (q, l)
                updateCollections ((g, n):rs) q l = case n of
                                                        OK -> updateCollections rs (pushToQueue q g) l
                                                        Lock _ -> updateCollections rs (pushToQueue q g) l
                                                        GameOver -> updateCollections rs q (pushToList l g)
                                                        _ -> updateCollections rs q l
                pushToQueue q x = PQ.insert (Game.score x) x q
                pushToList c x = x : c
